@@ -196,12 +196,11 @@ class OvsPlugin(plugin.PluginBase):
         return profile.datapath_type
 
     def _plug_vhostuser(self, vif, instance_info):
-        self.ovsdb.ensure_ovs_bridge(
-            vif.network.bridge, self._get_vif_datapath_type(
-                vif, datapath=constants.OVS_DATAPATH_NETDEV))
         vif_name = OvsPlugin.gen_port_name(
             constants.OVS_VHOSTUSER_PREFIX, vif.id)
         args = {}
+        args['datapath_type'] = self._get_vif_datapath_type(vif,
+                datapath=constants.OVS_DATAPATH_NETDEV)
         if vif.mode == "client":
             args['interface_type'] = \
                 constants.OVS_VHOSTUSER_INTERFACE_TYPE
@@ -358,13 +357,21 @@ class OvsPlugin(plugin.PluginBase):
                 vif=vif,
                 err="This vif type is not supported by this plugin")
 
+    def _is_trunk_bridge(self, bridge_name):
+        return bridge_name.startswith(constants.TRUNK_BR_PREFIX)
+
+    def _delete_bridge_if_trunk(self, vif):
+        if self._is_trunk_bridge(vif.network.bridge):
+            self.ovsdb.delete_ovs_bridge(vif.network.bridge)
+
     def _unplug_vhostuser(self, vif, instance_info):
         self.ovsdb.delete_ovs_vif_port(vif.network.bridge,
             OvsPlugin.gen_port_name(
                 constants.OVS_VHOSTUSER_PREFIX,
                 vif.id))
+        self._delete_bridge_if_trunk(vif)
 
-    def _unplug_bridge(self, vif, instance_info):
+    def _unplug_bridge(self, vif, instance_info, linux_bridge_name):
         """UnPlug using hybrid strategy
 
         Unhook port from OVS, unhook port from bridge, delete
@@ -373,14 +380,16 @@ class OvsPlugin(plugin.PluginBase):
 
         v1_name, v2_name = self.get_veth_pair_names(vif)
 
-        linux_net.delete_bridge(vif.bridge_name, v1_name)
+        linux_net.delete_bridge(linux_bridge_name, v1_name)
 
         self.ovsdb.delete_ovs_vif_port(vif.network.bridge, v2_name)
+        self._delete_bridge_if_trunk(vif)
 
     def _unplug_vif_windows(self, vif, instance_info):
         """Remove port from OVS."""
         self.ovsdb.delete_ovs_vif_port(vif.network.bridge, vif.id,
                                        delete_netdev=False)
+        self._delete_bridge_if_trunk(vif)
 
     def _unplug_port_bridge(self, vif, instance_info):
         """Create a per-VIF OVS bridge and patch pair."""
@@ -393,6 +402,7 @@ class OvsPlugin(plugin.PluginBase):
         self.ovsdb.delete_ovs_vif_port(port_bridge_name, port_bridge_patch)
         self.ovsdb.delete_ovs_vif_port(port_bridge_name, vif.vif_name)
         self.ovsdb.delete_ovs_bridge(port_bridge_name)
+        self._delete_bridge_if_trunk(vif)
 
     def _unplug_vif_generic(self, vif, instance_info):
         """Remove port from OVS."""
@@ -400,6 +410,7 @@ class OvsPlugin(plugin.PluginBase):
         # Iaf15fa7a678ec2624f7c12f634269c465fbad930 this should be correct
         # so this is not removed.
         self.ovsdb.delete_ovs_vif_port(vif.network.bridge, vif.vif_name)
+        self._delete_bridge_if_trunk(vif)
 
     def _unplug_vf(self, vif):
         """Remove port from OVS."""
@@ -421,6 +432,7 @@ class OvsPlugin(plugin.PluginBase):
             vif.network.bridge, representor, delete_netdev=False)
         if datapath == constants.OVS_DATAPATH_SYSTEM:
             linux_net.set_interface_state(representor, 'down')
+        self._delete_bridge_if_trunk(vif)
 
     def unplug(self, vif, instance_info):
         if not hasattr(vif, "port_profile"):
@@ -440,9 +452,13 @@ class OvsPlugin(plugin.PluginBase):
             if self.config.per_port_bridge:
                 self._unplug_port_bridge(vif, instance_info)
             else:
-                self._unplug_vif_generic(vif, instance_info)
+                linux_bridge_name = self.gen_port_name('qbr', vif.id)
+                if ip_lib.exists(linux_bridge_name):
+                    self._unplug_bridge(vif, instance_info, linux_bridge_name)
+                else:
+                    self._unplug_vif_generic(vif, instance_info)
         elif isinstance(vif, objects.vif.VIFBridge):
-            self._unplug_bridge(vif, instance_info)
+            self._unplug_bridge(vif, instance_info, vif.bridge_name)
         elif isinstance(vif, objects.vif.VIFVHostUser):
             self._unplug_vhostuser(vif, instance_info)
         elif isinstance(vif, objects.vif.VIFHostDevice):
