@@ -44,14 +44,10 @@ class BaseOVS(object):
     def _ovs_supports_mtu_requests(self):
         return self.ovsdb.has_table_column('Interface', 'mtu_request')
 
-    def _set_mtu_request(self, txn, dev, mtu):
-        txn.add(
-            self.ovsdb.db_set(
-                'Interface', dev, ('mtu_request', mtu)
-            )
-        )
+    def _set_mtu_request(self, dev, mtu):
+        self.ovsdb.db_set('Interface', dev, ('mtu_request', mtu)).execute()
 
-    def update_device_mtu(self, txn, dev, mtu, interface_type=None):
+    def update_device_mtu(self, dev, mtu, interface_type=None):
         if not mtu:
             return
         if interface_type not in [
@@ -64,7 +60,7 @@ class BaseOVS(object):
                 # programming the MTU and fallback to DHCP advertisement.
                 linux_net.set_device_mtu(dev, mtu)
         elif self._ovs_supports_mtu_requests():
-            self._set_mtu_request(txn, dev, mtu)
+            self._set_mtu_request(dev, mtu)
         else:
             LOG.debug("MTU not set on %(interface_name)s interface "
                       "of type %(interface_type)s.",
@@ -142,7 +138,7 @@ class BaseOVS(object):
     def create_ovs_vif_port(
         self, bridge, dev, iface_id, mac, instance_id,
         mtu=None, interface_type=None, vhost_server_path=None,
-        tag=None, pf_pci=None, vf_num=None, set_ids=True
+        tag=None, pf_pci=None, vf_num=None, set_ids=True, datapath_type=None
     ):
         """Create OVS port
 
@@ -158,6 +154,7 @@ class BaseOVS(object):
         :param pf_pci: PCI address of PF for dpdk representor port.
         :param vf_num: VF number of PF for dpdk representor port.
         :param set_ids: set external ids on port (bool).
+        :param datapath_type: datapath type for port's bridge
 
         .. note:: create DPDK representor port by setting all three values:
             `interface_type`, `pf_pci` and `vf_num`. if interface type is
@@ -181,20 +178,18 @@ class BaseOVS(object):
             col_values.append(('options',
                               {'dpdk-devargs': devargs_string}))
         with self.ovsdb.transaction() as txn:
+            if datapath_type:
+                txn.add(self.ovsdb.add_br(bridge, may_exist=True,
+                                          datapath_type=datapath_type))
             txn.add(self.ovsdb.add_port(bridge, dev))
             if tag:
                 txn.add(self.ovsdb.db_set('Port', dev, ('tag', tag)))
             if col_values:
                 txn.add(self.ovsdb.db_set('Interface', dev, *col_values))
-            self.update_device_mtu(
-                txn, dev, mtu, interface_type=interface_type
-            )
+        self.update_device_mtu(dev, mtu, interface_type=interface_type)
 
     def update_ovs_vif_port(self, dev, mtu=None, interface_type=None):
-        with self.ovsdb.transaction() as txn:
-            self.update_device_mtu(
-                txn, dev, mtu, interface_type=interface_type
-            )
+        self.update_device_mtu(dev, mtu, interface_type=interface_type)
 
     def delete_ovs_vif_port(self, bridge, dev, delete_netdev=True):
         self.ovsdb.del_port(dev, bridge=bridge, if_exists=True).execute()
