@@ -51,6 +51,12 @@ class PluginTest(testtools.TestCase):
             subnets=self.subnets,
             vlan=99)
 
+        self.network_ovs_trunk = objects.network.Network(
+            id='437c6db5-4e6f-4b43-b64b-ed6a11ee5ba7',
+            bridge='%s01' % constants.TRUNK_BR_PREFIX,
+            subnets=self.subnets,
+            vlan=99)
+
         self.network_ovs_mtu = objects.network.Network(
             id='437c6db5-4e6f-4b43-b64b-ed6a11ee5ba7',
             bridge='br0',
@@ -66,6 +72,7 @@ class PluginTest(testtools.TestCase):
             interface_id='e65867e0-9340-4a7f-a256-09af6eb7a3aa',
             datapath_type='system')
 
+        # This is used for ironic with vif_type=smartnic
         self.profile_ovs_smart_nic = objects.vif.VIFPortProfileOpenVSwitch(
             interface_id='e65867e0-9340-4a7f-a256-09af6eb7a3aa',
             create_port=True)
@@ -89,6 +96,7 @@ class PluginTest(testtools.TestCase):
             vif_name='tap-xxx-yyy-zzz',
             port_profile=self.profile_ovs)
 
+        # This is used for ironic with vif_type=smartnic
         self.vif_ovs_smart_nic = objects.vif.VIFOpenVSwitch(
             id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
             address='ca:fe:de:ad:be:ef',
@@ -104,6 +112,14 @@ class PluginTest(testtools.TestCase):
             mode='client',
             port_profile=self.profile_ovs)
 
+        self.vif_vhostuser_trunk = objects.vif.VIFVHostUser(
+            id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
+            address='ca:fe:de:ad:be:ef',
+            network=self.network_ovs_trunk,
+            path='/var/run/openvswitch/vhub679325f-ca',
+            mode='client',
+            port_profile=self.profile_ovs)
+
         self.vif_vhostuser_client = objects.vif.VIFVHostUser(
             id='b679325f-ca89-4ee0-a8be-6db1409b69ea',
             address='ca:fe:de:ad:be:ef',
@@ -153,7 +169,7 @@ class PluginTest(testtools.TestCase):
             self.vif_ovs.network.bridge, mock.sentinel.vif_name,
             self.vif_ovs.port_profile.interface_id,
             self.vif_ovs.address, self.instance.uuid,
-            plugin.config.network_device_mtu,
+            mtu=plugin.config.network_device_mtu,
             interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'create_ovs_vif_port')
@@ -167,7 +183,7 @@ class PluginTest(testtools.TestCase):
             self.vif_ovs.network.bridge, mock.sentinel.vif_name,
             self.vif_ovs.port_profile.interface_id,
             self.vif_ovs.address, self.instance.uuid,
-            self.network_ovs_mtu.mtu,
+            mtu=self.network_ovs_mtu.mtu,
             interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE)
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'create_ovs_vif_port')
@@ -181,18 +197,28 @@ class PluginTest(testtools.TestCase):
                 self.vif_ovs.network.bridge, mock.sentinel.vif_name,
                 self.vif_ovs.port_profile.interface_id,
                 self.vif_ovs.address, self.instance.uuid,
-                plugin.config.network_device_mtu,
+                mtu=plugin.config.network_device_mtu,
                 interface_type=constants.OVS_VHOSTUSER_INTERFACE_TYPE,
                 tag=constants.DEAD_VLAN)
 
     @mock.patch.object(ovs, 'sys')
     @mock.patch.object(ovs.OvsPlugin, '_plug_vif_generic')
-    def test_plug_ovs(self, plug_vif_generic, mock_sys):
+    def test_plug_ovs_port_bridge_false(self, plug_vif_generic, mock_sys):
         mock_sys.platform = 'linux'
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
-        plugin.plug(self.vif_ovs, self.instance)
-        plug_vif_generic.assert_called_once_with(self.vif_ovs,
-                                                 self.instance)
+        with mock.patch.object(plugin.config, 'per_port_bridge', False):
+            plugin.plug(self.vif_ovs, self.instance)
+            plug_vif_generic.assert_called_once_with(
+                self.vif_ovs, self.instance)
+
+    @mock.patch.object(ovs, 'sys')
+    @mock.patch.object(ovs.OvsPlugin, '_plug_port_bridge')
+    def test_plug_ovs_port_bridge_true(self, plug_vif, mock_sys):
+        mock_sys.platform = 'linux'
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        with mock.patch.object(plugin.config, 'per_port_bridge', True):
+            plugin.plug(self.vif_ovs, self.instance)
+            plug_vif.assert_called_once_with(self.vif_ovs, self.instance)
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, "_create_vif_port")
@@ -296,25 +322,44 @@ class PluginTest(testtools.TestCase):
     def test_plug_ovs_bridge_windows(self):
         self._check_plug_ovs_windows(self.vif_ovs_hybrid)
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs, 'sys')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
-    def test_unplug_ovs(self, unplug, mock_sys):
+    def test_unplug_ovs_port_bridge_false(self, unplug, mock_sys,
+                                          delete_ovs_bridge):
         mock_sys.platform = 'linux'
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
-        plugin.unplug(self.vif_ovs, self.instance)
-        unplug.assert_called_once_with(self.vif_ovs, self.instance)
+        with mock.patch.object(plugin.config, 'per_port_bridge', False):
+            plugin.unplug(self.vif_ovs, self.instance)
+            unplug.assert_called_once_with(self.vif_ovs, self.instance)
+        delete_ovs_bridge.assert_not_called()
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
+    @mock.patch.object(ovs, 'sys')
+    @mock.patch.object(ovs.OvsPlugin, '_unplug_port_bridge')
+    def test_unplug_ovs_port_bridge_true(self, unplug, mock_sys,
+                                         delete_ovs_bridge):
+        mock_sys.platform = 'linux'
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        with mock.patch.object(plugin.config, 'per_port_bridge', True):
+            plugin.unplug(self.vif_ovs, self.instance)
+            unplug.assert_called_once_with(self.vif_ovs, self.instance)
+        delete_ovs_bridge.assert_not_called()
+
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
-    def test_unplug_vif_generic(self, delete_port):
+    def test_unplug_vif_generic(self, delete_port, delete_ovs_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin._unplug_vif_generic(self.vif_ovs, self.instance)
         delete_port.assert_called_once()
+        delete_ovs_bridge.assert_not_called()
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
     @mock.patch.object(linux_net, 'delete_bridge')
     @mock.patch.object(ovs, 'sys')
     def test_unplug_ovs_bridge(self, mock_sys, delete_bridge,
-                               delete_ovs_vif_port):
+                               delete_ovs_vif_port, delete_ovs_bridge):
         calls = {
             'delete_bridge': [mock.call('qbrvif-xxx-yyy', 'qvbb679325f-ca')],
             'delete_ovs_vif_port': [mock.call('br0', 'qvob679325f-ca')]
@@ -324,6 +369,7 @@ class PluginTest(testtools.TestCase):
         plugin.unplug(self.vif_ovs_hybrid, self.instance)
         delete_bridge.assert_has_calls(calls['delete_bridge'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
     @mock.patch.object(ovs, 'sys')
@@ -334,62 +380,72 @@ class PluginTest(testtools.TestCase):
         delete_ovs_vif_port.assert_called_once_with('br0', vif.id,
                                                     delete_netdev=False)
 
-    def test_unplug_ovs_windows(self):
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
+    def test_unplug_ovs_windows(self, delete_ovs_bridge):
         self._check_unplug_ovs_windows(self.vif_ovs)
+        delete_ovs_bridge.assert_not_called()
 
-    def test_unplug_ovs_bridge_windows(self):
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
+    def test_unplug_ovs_bridge_windows(self, delete_ovs_bridge):
         self._check_unplug_ovs_windows(self.vif_ovs_hybrid)
+        delete_ovs_bridge.assert_not_called()
 
-    @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_create_vif_port')
-    def test_plug_ovs_vhostuser(self, _create_vif_port, ensure_ovs_bridge):
+    def test_plug_ovs_vhostuser(self, _create_vif_port):
         dp_type = ovs.OvsPlugin._get_vif_datapath_type(self.vif_vhostuser)
-        calls = {
-
-            '_create_vif_port': [mock.call(
-                                 self.vif_vhostuser, 'vhub679325f-ca',
-                                 self.instance,
-                                 interface_type='dpdkvhostuser')],
-            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
-        }
+        calls = [mock.call(
+                self.vif_vhostuser, 'vhub679325f-ca',
+                self.instance,
+                interface_type='dpdkvhostuser',
+                datapath_type=dp_type)]
 
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_vhostuser, self.instance)
-        _create_vif_port.assert_has_calls(calls['_create_vif_port'])
-        ensure_ovs_bridge.assert_has_calls(calls['ensure_ovs_bridge'])
+        _create_vif_port.assert_has_calls(calls)
 
-    @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'create_ovs_vif_port')
-    def test_plug_ovs_vhostuser_client(self, create_ovs_vif_port,
-                                       ensure_ovs_bridge):
+    def test_plug_ovs_vhostuser_client(self, create_ovs_vif_port):
         dp_type = ovs.OvsPlugin._get_vif_datapath_type(
             self.vif_vhostuser_client)
-        calls = {
-            'create_ovs_vif_port': [
+        calls = [
                  mock.call(
                      'br0', 'vhub679325f-ca',
                      'e65867e0-9340-4a7f-a256-09af6eb7a3aa',
                      'ca:fe:de:ad:be:ef',
                      'f0000000-0000-0000-0000-000000000001',
-                     1500, interface_type='dpdkvhostuserclient',
-                     vhost_server_path='/var/run/openvswitch/vhub679325f-ca'
-                 )],
-            'ensure_ovs_bridge': [mock.call('br0', dp_type)]
-        }
+                     mtu=1500, interface_type='dpdkvhostuserclient',
+                     vhost_server_path='/var/run/openvswitch/vhub679325f-ca',
+                     datapath_type=dp_type
+                 )]
 
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.plug(self.vif_vhostuser_client, self.instance)
-        create_ovs_vif_port.assert_has_calls(calls['create_ovs_vif_port'])
-        ensure_ovs_bridge.assert_has_calls(calls['ensure_ovs_bridge'])
+        create_ovs_vif_port.assert_has_calls(calls)
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
-    def test_unplug_ovs_vhostuser(self, delete_ovs_vif_port):
+    def test_unplug_ovs_vhostuser(self, delete_ovs_vif_port,
+                                  delete_ovs_bridge):
         calls = {
             'delete_ovs_vif_port': [mock.call('br0', 'vhub679325f-ca')]
         }
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
         plugin.unplug(self.vif_vhostuser, self.instance)
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_not_called()
+
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
+    def test_unplug_ovs_vhostuser_trunk(self, delete_ovs_vif_port,
+                                        delete_ovs_bridge):
+        bridge_name = '%s01' % constants.TRUNK_BR_PREFIX
+        calls = {
+            'delete_ovs_vif_port': [mock.call(bridge_name, 'vhub679325f-ca')]
+        }
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin.unplug(self.vif_vhostuser_trunk, self.instance)
+        delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_called_once_with(bridge_name)
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(linux_net, 'get_ifname_by_pci_address')
@@ -434,6 +490,7 @@ class PluginTest(testtools.TestCase):
         set_interface_state.assert_has_calls(calls['set_interface_state'])
         _create_vif_port.assert_has_calls(calls['_create_vif_port'])
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(linux_net, 'get_ifname_by_pci_address')
     @mock.patch.object(linux_net, 'get_vf_num_by_pci_address')
     @mock.patch.object(linux_net, 'get_representor_port')
@@ -443,7 +500,8 @@ class PluginTest(testtools.TestCase):
                                      set_interface_state,
                                      get_representor_port,
                                      get_vf_num_by_pci_address,
-                                     get_ifname_by_pci_address):
+                                     get_ifname_by_pci_address,
+                                     delete_ovs_bridge):
         calls = {
 
             'get_ifname_by_pci_address': [mock.call('0002:24:12.3',
@@ -469,20 +527,25 @@ class PluginTest(testtools.TestCase):
             calls['get_representor_port'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
         set_interface_state.assert_has_calls(calls['set_interface_state'])
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, "_create_vif_port")
-    def test_plug_vif_ovs_smart_nic(self, create_port, ensure_bridge):
+    def test_plug_vif_ovs_ironic_smart_nic(self, create_port, ensure_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
-        plugin.plug(self.vif_ovs_smart_nic, self.instance)
-        ensure_bridge.assert_called_once()
-        create_port.assert_called_once()
+        with mock.patch.object(plugin.config, 'per_port_bridge', False):
+            plugin.plug(self.vif_ovs_smart_nic, self.instance)
+            ensure_bridge.assert_called_once()
+            create_port.assert_called_once()
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
-    def test_unplug_vif_ovs_smart_nic(self, delete_port):
+    def test_unplug_vif_ovs_smart_nic(self, delete_port, delete_ovs_bridge):
         plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
-        plugin.unplug(self.vif_ovs_smart_nic, self.instance)
-        delete_port.assert_called_once()
+        with mock.patch.object(plugin.config, 'per_port_bridge', False):
+            plugin.unplug(self.vif_ovs_smart_nic, self.instance)
+            delete_port.assert_called_once()
+        delete_ovs_bridge.assert_not_called()
 
     @mock.patch.object(linux_net, 'get_dpdk_representor_port_name')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
@@ -528,10 +591,12 @@ class PluginTest(testtools.TestCase):
         _create_vif_port.assert_has_calls(
             calls['_create_vif_port'])
 
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
     @mock.patch.object(linux_net, 'get_dpdk_representor_port_name')
     @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
     def test_unplug_ovs_vf_dpdk(self, delete_ovs_vif_port,
-                                get_dpdk_representor_port_name):
+                                get_dpdk_representor_port_name,
+                                delete_ovs_bridge):
         devname = 'vfrb679325f-ca'
         get_dpdk_representor_port_name.return_value = devname
         calls = {
@@ -544,3 +609,48 @@ class PluginTest(testtools.TestCase):
         get_dpdk_representor_port_name.assert_has_calls(
             calls['get_dpdk_representor_port_name'])
         delete_ovs_vif_port.assert_has_calls(calls['delete_ovs_vif_port'])
+        delete_ovs_bridge.assert_not_called()
+
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'create_patch_port_pair')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'ensure_ovs_bridge')
+    @mock.patch.object(ovs.OvsPlugin, "_create_vif_port")
+    def test_plug_port_bridge(
+            self, create_port, ensure_bridge, create_patch_port_pair):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin._plug_port_bridge(self.vif_ovs, self.instance)
+        calls = [
+            mock.call('br0', 'netdev'),
+            mock.call('pbb679325f-ca8', 'netdev')
+        ]
+        ensure_bridge.assert_has_calls(calls)
+        create_port.assert_called_once()
+        create_patch_port_pair.assert_called_once()
+
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_vif_port')
+    @mock.patch.object(ovsdb_lib.BaseOVS, 'delete_ovs_bridge')
+    def test_unplug_port_bridge(
+            self, delete_ovs_bridge, delete_ovs_vif_port):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin._unplug_port_bridge(self.vif_ovs, self.instance)
+        calls = [
+            mock.call('br0', 'ibpb679325f-ca89-4ee0-a8be-6db1409b69ea'),
+            mock.call(
+                'pbb679325f-ca8', 'pbpb679325f-ca89-4ee0-a8be-6db1409b69ea'),
+            mock.call('pbb679325f-ca8', 'tap-xxx-yyy-zzz')
+        ]
+        delete_ovs_vif_port.assert_has_calls(calls)
+        delete_ovs_bridge.assert_called_once_with('pbb679325f-ca8')
+
+    @mock.patch.object(ip_lib, 'exists', return_value=True)
+    @mock.patch.object(ovs.OvsPlugin, '_unplug_bridge')
+    def test_unplug_hybrid_bridge(self, m_unplug_bridge, m_ip_lib_exists):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin.unplug(self.vif_ovs, self.instance)
+        m_unplug_bridge.assert_called_once()
+
+    @mock.patch.object(ip_lib, 'exists', return_value=False)
+    @mock.patch.object(ovs.OvsPlugin, '_unplug_vif_generic')
+    def test_unplug_ovs(self, m_unplug_generic, m_ip_lib_exists):
+        plugin = ovs.OvsPlugin.load(constants.PLUGIN_NAME)
+        plugin.unplug(self.vif_ovs, self.instance)
+        m_unplug_generic.assert_called_once()
